/**************************************************************************
*   Amarok 2 lyrics script to search for lyrics on music.goo.ne.jp        *
*                                                                         *
*   Copyright                                                             *
*   (C) 2008 Ignacio Serantes <kde@aynoa.net>                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
**************************************************************************/

/*
BUG: el input dialog aparece aveces antes de que acaben todas las búsquedas.
BUG: el input dialog no se cierra cuando se cambia de canción.

TODO: conversión entre escritura japonesa y occidental.
TODO: gestión de múltiples resultados catalogados por servidor.

*/

Importer.loadQtBinding("qt.core");
Importer.loadQtBinding("qt.gui");
//Importer.loadQtBinding("qt.network");
//Importer.loadQtBinding("qt.opengl");
//Importer.loadQtBinding("qt.sql");
//Importer.loadQtBinding("qt.svg");
//Importer.loadQtBinding("qt.uitools");
//Importer.loadQtBinding("qt.webkit");
//Importer.loadQtBinding("qt.xml");


//
// Funciones generales.
//

// Trim. Alucino que estas funciones no estén disponibles.
function trim(stringToTrim) {
  return stringToTrim.replace(/^\s+|\s+$/g, '');
}
function ltrim(stringToTrim) {
  return stringToTrim.replace(/^\s+/, '');
}
function rtrim(stringToTrim) {
  return stringToTrim.replace(/\s+$/, '');
}

// Pasamos un valor a hexadecimal.
function toHex(dec) {
  if (dec < 0) dec = 256 + dec;
  var hexCharacters = "0123456789abcdef";
  if (dec < 0) return "00";
  if (dec > 255) return "FF";
  var i = Math.floor(dec / 16);
  var j = dec % 16;
  return hexCharacters.charAt(i) + hexCharacters.charAt(j);
}

// Convierte un QByteArray a percent encoding.
function ba2p(ba) {
  var str = '';
  for (i = 0; i < ba.length(); i++) {
     str += '%' + toHex(ba.at(i));
  }
  return str;
}

// Conversión de un QByteArray a string.
function ba2s(ba) {
  var str = '';
  for (i = 0; i < ba.length(); i++) {
    str += String.fromCharCode(ba.at(i));
  }
  return str;
}

// Obtiene la información entre dos tags.
// usar ocurrence par ver con cual nos quedamos. Si se indica un
// nº mayor al de ocurrencias se devuelve la cadena vacía.
function getDataBetweenTags(data, beginTag, endTag, ocurrence) {

  var ocurrence = (ocurrence == null) ? 1 : ocurrence;
  var iBegin = data.indexOf(beginTag) + beginTag.length;
  var iEnd = data.indexOf(endTag, iBegin);

  ocurrence--;
  while ((ocurrence > 0) && (iBegin >= 0)) {
    iBegin = data.indexOf(beginTag, iEnd + endTag.length);
    if (iBegin > -1) {
       iBegin += beginTag.length;
       iEnd = data.indexOf(endTag, iBegin);
    } else {
       iEnd = -1;
    }
    ocurrence--;
  }

  if ( (iBegin >= 0) && (iBegin < iEnd)) {
    return data.substring(iBegin, iEnd);
  } else {
    return '';
  }

} // function getDataBetweenTags(data, beginTag, endTag, ocurrence)

// Elimina los comentarios del HTML.
function cleanHTMLComments(data) {

  var iBegin = 0;

  while (iBegin >= 0) {
    iBegin = data.indexOf('<!--');
    if (iBegin > -1) {
      data = data.substring(0, iBegin) + data.substring(data.indexOf('-->', iBegin) + 3);
    }
  }

  return data;

} // function cleanHTMLComments(data)

// Convierte (o al menos lo intenta) html a texto plano.
function html2text(html) {

  // Nos quedamos únicamente con el body.
  html = html.substring(html.indexOf('<body')); // No es correcto del todo pero funciona y es más rápido.

  // Eliminamos los comentarios.
  html = cleanHTMLComments(html);

  // Convertimos los \r en \n.
  html = html.replace(/\r\n/g, '\n');
  html = html.replace(/\r/g, '\n');

  // Eliminamos las líneas vacías del principio y del final.
  html = html.replace(/^\n+|\n+$/g, '');

  // Si no hay \n debemos cambiar los <br> por \n.
  if (html.indexOf('\n') < 0) { 
    html = html.replace(/<br>/g, '\n');
    html = html.replace(/<br \/>/g, '\n');
  }

  // Eliminamos todo lo que encontremos entre "<" y ">".
  html = html.replace(/<[^>]*>/g, '');
  html = html.replace(/<br \/>/g, '');

  // Convertimos a caracteres.
  html = html.replace(/&#(\d+);/g, function (m, n) { return String.fromCharCode(n); })

  return trim(html);

} // function html2text(html)

// Graba en el fichero indicado el texto indicado.
function saveTextFile(fileName, text) {

  var fs = new QFile(fileName);
  fs.open(new QIODevice.OpenMode(QIODevice.Truncate | QIODevice.WriteOnly | QIODevice.Text));
  var ts = new QTextStream(fs);
  ts.writeString(text);
  fs.flush();
  fs.close();

} // function saveTextFile(fileName, text)

// Crea el fichero específico para que sea leido por el plasmoide STDIN.
function saveToSTDIN(html) {

  var tmpDir = QDir.tempPath();
  var fileName = tmpDir + '/GOO_Lyrics_tmp.sh';
  var fs = new QFile(fileName);
  fs.open(new QIODevice.OpenMode(QIODevice.Truncate | QIODevice.WriteOnly | QIODevice.Text));
  var ts = new QTextStream(fs);
  html = html.replace(/\n/g, '<br />');
  html = html.replace(/"/g, '\\"');
  html = html.replace(/`/g, '\\`');
  //html = html.replace(/'/g, "\\'");
  ts.writeString('#!/bin/sh\n');
  //ts.writeString('echo -e "' + html + '" | html2text -utf8\n');
  ts.writeString('echo -e "' + html + '" | html2text -utf8 -nobs -style compact -o ' + tmpDir + '/GOO_Lyrics.txt\n');
  ts.writeString('rm "' + fileName + '"\n');
  fs.flush();
  fs.close();
  QProcess.execute('chmod +x "' + fileName + '"');
  QProcess.execute(fileName);

} // function saveToSTDIN(html)


//
// Funciones comunes.
//

// Grabamos la letra y la visualizamos.
function saveLyrics(html) {
  
  try {

    // Una comprobación final por si se llega aquí después de un cambio de canción.
    if ((initial_artist != Amarok.Engine.currentTrack().artist) || (initial_title != Amarok.Engine.currentTrack().title)) {
      return true;
    }

    Amarok.Lyrics.showLyrics(html); // Sólo los graba.
    Amarok.Lyrics.showLyricsHtml(html); // Sólo los muestra.
    if (enableSTDIN) {saveToSTDIN(html)};

  } catch(err) {
        Amarok.debug( "error: " + err );
  }

} // function saveLyrics(html)

// Formateamos para Amarok.
function formatHTML(artist, title, urlArtist, urlLyrics, server, urlServer, lyrics) {

  try {
  
    var pageTitle = title + " " + artist + " lyrics";
    var footerFrom = (server == '') ? '' : '<strong>fetched from:</strong> <a href="' + urlServer + '">' + server + '</a>\n';
    var footerLyrics = (urlLyrics == '') ? '' : '<strong>lyrics:</strong> <a href="' + urlLyrics + '">' + urlLyrics + '</a>\n';
    var footerArtist = (urlArtist == '') ? '' : '<strong>artist:</strong> <a href="' + urlArtist + '">' + urlArtist + '</a>\n';
    
    // OJO: no hacerlo con html.replace ya que algunos carácteres en la letra, por ejemplo $10 
    // tienen significado y la sustitución hace cosas raras.
    var html = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">\n<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en" dir="ltr">\n<head>\n<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />\n<title>' + pageTitle + '</title>\n</head>\n<body><h3><a href="' + urlLyrics + '">' + title + '</a> by <a href="' + urlArtist + '">' + artist + '</a></h3>\n<pre>\n' + lyrics + '</pre><hr/><pre><a href="">' + TEXT_HTML_ADDITIONAL_INFO + footerFrom + footerLyrics + footerArtist + '</pre></body>\n</html>';

  } catch(err) {
        Amarok.debug( "error: " + err );
  }

  return html;

} // function formatHTML(artist, title, urlArtist, urlLyrics, server, urlServer, lyrics)

// Input dialog.
function titleArtistInputDialog(msg, artist, title) {

  return QInputDialog.getText(null, TEXT_SCRIPT_NAME, msg + TEXT_INPUT_DIALOG_HELP, QLineEdit.Normal, '"' + title + '" - "' + artist + '"', 0);

} // function inputBox()


// Al finalizar la descarga.
function onDownloadFinished(data) {
  try {

    if (data == '') { return true; }

    for (i = 0; i < SEARCH_ENGINES; i++) {
      
      // Comprobamos si tiene la marca adecuada.
      if (data.indexOf(searchEngines[i][DATA_SERVER_MARK]) >= 0) {

        // Incrementamos el contador para determinar cuando ha fallado la búsqueda.
        hitsCount += 1;

        // Verificamos que sean datos válidos.
        if (data.indexOf(NOT_FOUND_MARK) == -1) {

          if (searchEngines[i][DATA_SERVER_PRIORITY] > lastPriority) { return true};

          data = getDataBetweenTags(data, searchEngines[i][DATA_HTML_TAG_BEGIN], searchEngines[i][DATA_HTML_TAG_END], searchEngines[i][DATA_HTML_TAG_OCURRENCE]);
          
          if (data != '') {
            //saveTextFile(Amarok.Info.scriptPath() + '/goo.html', data);            
            Amarok.Window.Statusbar.shortMessage(searchEngines[i][DATA_FOUND_STRING]);
            lastPriority = searchEngines[i][DATA_SERVER_PRIORITY];
            saveLyrics(formatHTML(initial_artist, initial_title, searchGlobals[i][GLOBALS_ARTIST], searchGlobals[i][GLOBALS_LYRIC], searchEngines[i][DATA_SERVER_NAME], searchEngines[i][DATA_SERVER_URL], html2text(data)));
            return true;
          }
        
        } // if (data.indexOf(NOT_FOUND_MARK) == -1)
          
        // TODO: Bug de sincronización entre threads. Debo buscar una solución a este problema.
        if ((hitsCount == SEARCH_ENGINES) && (lastPriority == LOWEST_PRIORITY)) {

          var newData = null;
          if (cfg_inputBoxEnabled) { newData = titleArtistInputDialog(TEXT_MSG_TRY_AGAIN, global_artist, global_title); };

          if (newData == null) {

            if (html2text(Amarok.Engine.currentTrack().lyrics) == '') {
              Amarok.Window.Statusbar.shortMessage(TEXT_MSG_NOT_FOUND);
              saveLyrics(formatHTML(initial_artist, initial_title, '', '', '', '', TEXT_SONG_NOT_FOUND));
            }

          } else {

            if (newData == '') {
              // Instrumental.
              Amarok.Window.Statusbar.shortMessage(TEXT_MSG_NOT_EXISTS);
              saveLyrics(formatHTML(initial_artist, initial_title, '', '', '', '', TEXT_SONG_INSTRUMENTAL));
            } else {
              newTitle = trim(getDataBetweenTags(newData, '"', '" - "', 1));
              newArtist = trim(getDataBetweenTags(newData, '" - "', '"', 1));
              getLyrics(newArtist, newTitle, FLAG_RETRY);
            } // if (newData == '')

          } // if (newData == null)
        } // if ((hitsCount == SEARCH_ENGINES) && (lastPriority == 99))

        return true;

      } // if (data.indexOf(searchEngines[i, 2]) >= )
    } // for (i = 0; i <= SEARCH_ENGINES; i++)

  } catch(err) {
        Amarok.debug( "error: " + err );
  }

} // function onDownloadFinished(data)


//
// Funciones de gestión de mp3.sogou.com
//

// Examina la página de canción-artista y obtiene la url de la letras
// y la del artista.
function extractUrlsFrom_sogou(html) {

  try {

    // Por el momento dejamos esto en blanco.
    urlLyrics = '';
    urlArtist = '';
    
    //saveTextFile(Amarok.Info.scriptPath() + '/goo.html', html);
    //searchFails = (html.indexOf('<div class="noresult">') >= 0);
    //searchFails = false;
    sogouTitle = trim(html2text(getDataBetweenTags(html, '<b>曲名：', '</b>', 1)));
    //Amarok.alert(sogouTitle);
    sogouArtist = trim(html2text(getDataBetweenTags(html, '<b>歌手：', '</b>', 1)));
    //Amarok.alert(sogouArtist);

    if ((sogouTitle.toLowerCase() != global_title.toLowerCase()) || (sogouArtist.toLowerCase() != global_artist.toLowerCase())) {

      // No se encontró.
      onDownloadFinished(searchEngines[SEARCH_ENGINE_SOGOU][DATA_SERVER_URL] + NOT_FOUND_MARK);

    } else {

      //Amarok.alert('match');
      searchGlobals[SEARCH_ENGINE_SOGOU][GLOBALS_LYRIC] = urlLyrics;
      searchGlobals[SEARCH_ENGINE_SOGOU][GLOBALS_ARTIST] = urlArtist;
      onDownloadFinished(html);

    }

  } catch(err) {
    Amarok.debug( "error: " + err );
  }

  return true;

} // function extractUrlsFrom_sogou(html)

// Crear la url de búsqueda para descargar los datos.
function search_sogou(artist, title) {

  // Chinese encodings GB18030/GBK/GB2312, GB2312 codepage 936.
  // http://doc.trolltech.com/4.4/codec-gbk.html
  encodedArtist = new QByteArray(Amarok.Lyrics.fromUtf8(artist, "GB2312"));
  encodedTitle = new QByteArray(Amarok.Lyrics.fromUtf8(title, "GB2312"));
  var url = new QUrl(searchEngines[SEARCH_ENGINE_SOGOU][DATA_SERVER_SEARCH] + "/gecisearch.so");
  url.setEncodedQuery(new QByteArray('query="' + ba2p(encodedTitle) + "'+'" + ba2p(encodedArtist) + '"'));
  new Downloader(url, extractUrlsFrom_sogou, 'GB2312');
  return true;

} // function search_sogou(artist, title)



//
// Funciones de gestión de cix.co.kr
//

// Examina la página de canción-artista y obtiene la url de la letras
// y la del artista.
function extractUrlsFrom_cix(html) {

  try {

    // Las letras en la primera ocurrencia de href y el artista en la segunda.
    // Hay tantas filas como resultados pero por el momento nos quedamos con
    // el primero válido.
    urlLyrics = getDataBetweenTags(html, '<A HREF=', ' target=_top>', 1);
    urlArtist = (urlLyrics == '') ? '' : getDataBetweenTags(html, '<A HREF=', ' target=_top>', 2);

    if (urlLyrics == '') {

      // No se encontró.
      onDownloadFinished(searchEngines[SEARCH_ENGINE_CIX][DATA_SERVER_URL] + NOT_FOUND_MARK);

    } else {

      searchGlobals[SEARCH_ENGINE_CIX][GLOBALS_LYRIC] = searchEngines[SEARCH_ENGINE_CIX][DATA_SERVER_SEARCH] + '/kr/' + urlLyrics;
      searchGlobals[SEARCH_ENGINE_CIX][GLOBALS_ARTIST] = searchEngines[SEARCH_ENGINE_CIX][DATA_SERVER_SEARCH] + '/kr/' + urlArtist;
      var url = new QUrl(searchGlobals[SEARCH_ENGINE_CIX][GLOBALS_LYRIC]);
      new Downloader(url, onDownloadFinished, 'EUC-KR');

    }

  } catch(err) {
    Amarok.debug( "error: " + err );
  }

  return true;

} // function extractUrlsFrom_cix(html)


// Crear la url de búsqueda para descargar los datos.
function search_cix(artist, title) {

  encodedArtist = new QByteArray(Amarok.Lyrics.fromUtf8(artist, "EUC-KR"));
  encodedTitle = new QByteArray(Amarok.Lyrics.fromUtf8(title, "EUC-KR"));
  var url = new QUrl(searchEngines[SEARCH_ENGINE_CIX][DATA_SERVER_SEARCH] + "/kr/search_lyrics_songlist_main.html");
  url.setEncodedQuery(new QByteArray("SearchSong=" + ba2p(encodedTitle) + "&SearchSinger=" + ba2p(encodedArtist) + "&SearchContents=&SearchType=Start&btnSubmit="));
  new Downloader(url, extractUrlsFrom_cix, 'EUC-KR');
  return true;

} // function search_cix(artist, title)


//
// Funciones de gestión de evesta.jp
//

// Examina la página de canción-artista y obtiene la url de la letras
// y la del artista.
function extractUrlsFrom_evesta(html) {

  try {

    // La lista de resultados se encuentra entre "<div id='lyricList'>" y "</div>".
    html = getDataBetweenTags(html, '<div id=\'lyricList\'>', '</div>', 1);
    var data = '';
    var ocurrence = 1;
    var currTitle = '';
    var currArtist = '';
    var urlLyrics = '';
    var urlArtist = '';
    while (true) {

      data = getDataBetweenTags(html, '\' onmouseover=\'this.className="selected"\' onmouseout=\'this.className="', '</tr>', ocurrence);
    
      // No hay más registros.
      if (data == '') { break; };

      // El título está en la primera ocurrencia "<a href></a>".
      currTitle = trim(getDataBetweenTags(getDataBetweenTags(data, '<a href=', '/a>', 1), ">", "<", 1).toLowerCase());
      // El autor está en la segunda ocurrencia "<a href></a>".
      currArtist = trim(getDataBetweenTags(getDataBetweenTags(data, '<a href=', '/a>', 2), ">", "<", 1).toLowerCase());
      if ((currTitle == initial_title.toLowerCase()) && (currArtist == initial_artist.toLowerCase())) {
        urlLyrics = getDataBetweenTags(data, "<a href='", "'>", 1);
        urlArtist = getDataBetweenTags(data, "<a href='", "'>", 2);
        break;
      } // if ((currTitle == initial_title) || (currArtist == initial_artist))
      
      ocurrence++;

    } // while (true)

    if (urlLyrics == '' ) {

      // No se encontró.
      onDownloadFinished(searchEngines[SEARCH_ENGINE_EVESTA][DATA_SERVER_URL] + NOT_FOUND_MARK);

    } else {

      searchGlobals[SEARCH_ENGINE_EVESTA][GLOBALS_LYRIC] = urlLyrics;
      searchGlobals[SEARCH_ENGINE_EVESTA][GLOBALS_ARTIST] = urlArtist;
      var url = new QUrl(searchGlobals[SEARCH_ENGINE_EVESTA][GLOBALS_LYRIC]);
      new Downloader(url, onDownloadFinished);

    }

  } catch(err) {
    Amarok.debug( "error: " + err );
  }

  return true;

} // function extractUrlsFrom_evesta(html)

// Crear la url de búsqueda para descargar los datos.
function search_evesta(artist, title) {
  
  // Lo que me costó conseguir estas líneas :(.
  var url = new QUrl(searchEngines[SEARCH_ENGINE_EVESTA][DATA_SERVER_SEARCH] + "/lyric/search.php");
  url.addQueryItem('o', 's');
  url.addQueryItem('k', title + '+' + artist);
  url.addQueryItem('c', '0');
  url.addQueryItem('search', '検索');
  new Downloader(url, extractUrlsFrom_evesta);
  return true;

} // function search_evesta(artist, title)


//
// Funciones de gestión de music.goo.ne.jp
//

// Examina la página de canción-artista y obtiene la url de la letras
// y la del artista.
function extractUrlsFrom_goo(html) {

  try {

    // Las letras se encuentran en la primera ocurrencia de <div class="lst1">.
    var data = getDataBetweenTags(html, '<div class="lst1">', '</div>', 1);
    urlLyrics = getDataBetweenTags(data, '<a href="', '">', 1);
    urlArtist = getDataBetweenTags(data, '<a href="', '">', 2);

    if (urlLyrics == searchEngines[SEARCH_ENGINE_GOO][DATA_SERVER_URL]) {

      // No se encontró.
      onDownloadFinished(urlLyrics + NOT_FOUND_MARK);

    } else {

      searchGlobals[SEARCH_ENGINE_GOO][GLOBALS_LYRIC] = searchEngines[SEARCH_ENGINE_GOO][DATA_SERVER_SEARCH] + urlLyrics;
      searchGlobals[SEARCH_ENGINE_GOO][GLOBALS_ARTIST] = searchEngines[SEARCH_ENGINE_GOO][DATA_SERVER_SEARCH] + urlArtist;
      var url = new QUrl(searchGlobals[SEARCH_ENGINE_GOO][GLOBALS_LYRIC]);
      new Downloader(url, onDownloadFinished, 'EUC-JP');

    }

  } catch(err) {
    Amarok.debug( "error: " + err );
  }

  return true;

} // function extractUrlsFrom_goo(html)

// Crear la url de búsqueda para descargar los datos.
function search_goo(artist, title) {

  // Lo que me costó conseguir estas líneas :(.
  encodedArtist = new QByteArray(Amarok.Lyrics.fromUtf8(artist, "EUC-JP"));
  encodedTitle = new QByteArray(Amarok.Lyrics.fromUtf8(title, "EUC-JP"));
  var url = new QUrl(searchEngines[SEARCH_ENGINE_GOO][DATA_SERVER_SEARCH] + "/lyric/db.php");
  url.setEncodedQuery(new QByteArray("a=" + ba2p(encodedArtist) + "&k=" + ba2p(encodedTitle) + "&l=&s=&c=&submit="));
  new Downloader(url, extractUrlsFrom_goo, 'EUC-JP');
  return true;

} // function search_goo(artist, title)


//
// Funciones de gestión de LyricWiki.org
//

// Comprobamos que la página tiene datos.
function parseData_lyricWiki(html) {

  try {

    // Las letras en la primera ocurrencia de href y el artista en la segunda.
    urlLyrics = getDataBetweenTags(html, "<a href='", "'>", 1);
    urlArtist = getDataBetweenTags(html, "<a href='", "'>", 2);

    rawLyrics = html2text(getDataBetweenTags(html, searchEngines[SEARCH_ENGINE_LYRICWIKI][DATA_HTML_TAG_BEGIN], searchEngines[SEARCH_ENGINE_LYRICWIKI][DATA_HTML_TAG_END], searchEngines[SEARCH_ENGINE_LYRICWIKI][DATA_HTML_TAG_OCURRENCE]));
    
    if (rawLyrics == 'Not found') {

      // No se encontró.
      onDownloadFinished(searchEngines[SEARCH_ENGINE_LYRICWIKI][DATA_SERVER_URL] + NOT_FOUND_MARK);

    } else {

      searchGlobals[SEARCH_ENGINE_LYRICWIKI][GLOBALS_LYRIC] = urlLyrics;
      searchGlobals[SEARCH_ENGINE_LYRICWIKI][GLOBALS_ARTIST] = urlArtist;
      //Amarok.alert(html);
      onDownloadFinished(html);

    }

  } catch(err) {
    Amarok.debug( "error: " + err );
  }

  return true;

} // function parseData_lyricWiki(html)

// Crear la url de búsqueda para descargar los datos.
function search_lyricWiki(artist, title) {

  var url = new QUrl(searchEngines[SEARCH_ENGINE_LYRICWIKI][DATA_SERVER_SEARCH] + "/api.php");
  url.addQueryItem("func", "getSong");
  url.addQueryItem("artist", artist);
  url.addQueryItem("song", title);
  new Downloader(url, parseData_lyricWiki);
  return true;

} // function search_lyricWiki(artist, title)


// Configuración.
function configureGOOlyrics() {

  try {

    var items = new Array();
    items[0] = (cfg_inputBoxEnabled) ? TEXT_MSG_STATE_ENABLED : TEXT_MSG_STATE_DISABLED;
    items[1] = (cfg_inputBoxEnabled) ? TEXT_MSG_STATE_DISABLED : TEXT_MSG_STATE_ENABLED;

    var item = QInputDialog.getItem(null, TEXT_CFG_TITLE, TEXT_CFG_INPUT_DIALOG, items, 0, false, 0);

    if (item == null) { return true; };

    var newValue = (item == TEXT_MSG_STATE_ENABLED) ? true : false;

    if (newValue != cfg_inputBoxEnabled) {
    
      // Cambio la configuración.
      if (newValue) {
        
        // Creamos el fichero.
        var fs = new QFile(Amarok.Info.scriptPath() + '/' + CFG_FILE_NAME);
        fs.open(new QIODevice.OpenMode(QIODevice.Truncate | QIODevice.WriteOnly | QIODevice.Text));
        var ts = new QTextStream(fs);
        ts.writeString('// ' + TEXT_SCRIPT_NAME + ' configuration file');
        fs.flush();
        fs.close();

        cfg_inputBoxEnabled = true;

        Amarok.alert('This option is in alpha stage and "input dialog" may block Amarok.\n\nPlease be careful and don\'t forget report bugs to "kde@aynoa.net".');

      } else {

        // Borramos el fichero.
        QFile.remove(Amarok.Info.scriptPath() + '/' + CFG_FILE_NAME);

        cfg_inputBoxEnabled = false;

      } // if (newValue) 

    } // if (newValue != cfg_inputBoxEnabled)

  } catch(err) {
    Amarok.debug( "error: " + err );
  }

  return true;

} // function configureGOOlyrics()


//
// Función principal.
//
function getLyrics(artist, title, url) {
  try {

    // Inicializamos variables globales.
    if (url != FLAG_RETRY) {
      // Puede venir de askForLyrics así que no nos podemos fiar
      // de los parámetros artist y title.
      initial_artist = trim(Amarok.Engine.currentTrack().artist);
      initial_title = trim(Amarok.Engine.currentTrack().title);
    } else {
      // Para que no interfiera más abajo FLAG_RETRY se debe
      // interpretar como si se pasara una url vacía.
      url = '';
    }

    global_artist = artist;
    global_title = title;
    searchGlobals[SEARCH_ENGINE_GOO] = new Array('', '');
    searchGlobals[SEARCH_ENGINE_CIX] = new Array('', '');
    searchGlobals[SEARCH_ENGINE_LYRICWIKI] = new Array('', '');
    searchGlobals[SEARCH_ENGINE_SOGOU] = new Array('', '');
    searchGlobals[SEARCH_ENGINE_EVESTA] = new Array('', '');

    // Estas variables gestionan la interacción entre los servidores
    // y en un futuro la precedencia entre ellos.
    lastPriority = LOWEST_PRIORITY;
    hitsCount = 0;

    if (url != '') {

      Amarok.Window.Statusbar.shortMessage(TEXT_MSG_REUSING);
      var html = Amarok.Engine.currentTrack().lyrics;
      Amarok.Lyrics.showLyricsHtml(html);
      if (enableSTDIN) {saveToSTDIN(html)};

    } else {

      Amarok.Window.Statusbar.shortMessage(TEXT_MSG_SEARCHING);
      search_lyricWiki(artist, title);
      search_goo(artist, title);
      search_cix(artist, title);
      search_sogou(artist, title);
      search_evesta(artist, title);

    } // if (url != '')

  } catch(err) {
    Amarok.debug( "error: " + err );
   }

} // function getLyrics(artist, title, url)

function askForLyrics() {

  // Si amarok no está tocando no seguimos.
  if (Amarok.Engine.engineState() == AMAROK_ENGINE_STATE_STOP) { 
    Amarok.Window.Statusbar.longMessage(TEXT_MSG_OPTION_DISABLED);
    return true; 
  }

  var artist = Amarok.Engine.currentTrack().artist;
  var title = Amarok.Engine.currentTrack().title;
  var newData = titleArtistInputDialog(TEXT_MSG_SEARCH_WHAT, artist, title);

  if (newData != null) {

    if (newData == '') {
      // Instrumental.
      Amarok.Window.Statusbar.shortMessage(TEXT_MSG_NOT_EXISTS);
      saveLyrics(formatHTML(title, artist, '', '', '', '', TEXT_SONG_INSTRUMENTAL));
    } else {
      title = trim(getDataBetweenTags(newData, '"', '" - "', 1));
      artist = trim(getDataBetweenTags(newData, '" - "', '"', 1));
      getLyrics(artist, title, '');
    } // if (newData == '')

  } // if (newData == null)

} // function askForLyrics()


/*
function trackChange() {

  var html = Amarok.Engine.currentTrack().lyrics;

  if (html.length != '') {
    Amarok.Window.Statusbar.shortMessage('GOO extracting lyrics');
    if (enableSTDIN) {saveToSTDIN(html)};
  }

} // function trackChange() 
*/

//
// Constantes.
// 

// Array de searchEngines.
const SEARCH_ENGINE_GOO = 0;
const SEARCH_ENGINE_CIX = 1;
const SEARCH_ENGINE_LYRICWIKI = 2;
const SEARCH_ENGINE_SOGOU = 3;
const SEARCH_ENGINE_EVESTA = 4;
const SEARCH_ENGINES = 5;

// Array de engine, un elemento del array de searchEngines.
const DATA_SERVER_URL = 0;
const DATA_SERVER_SEARCH = 1;
const DATA_SERVER_MARK = 2;
const DATA_FOUND_STRING = 3;
const DATA_HTML_TAG_BEGIN = 4;
const DATA_HTML_TAG_END = 5;
const DATA_HTML_TAG_OCURRENCE = 6;
const DATA_SERVER_NAME = 7;
const DATA_SERVER_PRIORITY = 8;

// Array de searchGlobals.
const GLOBALS_LYRIC = 0;
const GLOBALS_ARTIST = 1;

// Otras constantes.
const CFG_FILE_NAME = 'GOO_lyrics.cfg';
const FLAG_RETRY = '<<RETRY>>';
const LOWEST_PRIORITY = 999;
const NOT_FOUND_MARK = '<pre>Not found</pre>';

// Constantes de amarok.
//const AMAROK_ENGINE_STATE_PLAY = 0;
//const AMAROK_ENGINE_STATE_PAUSE = 1;
const AMAROK_ENGINE_STATE_STOP = 2;

// Mensajes.
const TEXT_SCRIPT_NAME = 'GOO lyrics';

const TEXT_CFG_INPUT_DIALOG = 'Behavior of the input dialog if the search fails:';
const TEXT_CFG_TITLE = 'Configure GOO lyrics';
const TEXT_HTML_ADDITIONAL_INFO = '<strong>GOO lyrics</strong></a> additional information:\n';
const TEXT_INPUT_DIALOG_HELP = '\n\nTips:\n - format is "Title" - "Artist"\n - respect format if you want search works\n - empty data and "OK" for instrumental songs\n - "Cancel" buttom aborts search';
const TEXT_MSG_NOT_EXISTS = 'GOO lyrics don\'t exists :)';
const TEXT_MSG_NOT_FOUND = 'GOO lyrics not found :(';
const TEXT_MSG_OPTION_DISABLED = 'Option disabled if Amarok is not playing.';
const TEXT_MSG_REUSING = 'GOO reusing lyrics';
const TEXT_MSG_SEARCH_WHAT = 'What do you want to search?';
const TEXT_MSG_SEARCHING = 'GOO searching...';
const TEXT_MSG_STATE_ENABLED = 'enabled';
const TEXT_MSG_STATE_DISABLED = 'disabled';
const TEXT_MSG_TRY_AGAIN = 'Should I try another search?';
const TEXT_SONG_INSTRUMENTAL = 'Instrumental';
const TEXT_SONG_NOT_FOUND = 'Not found';

// Esto debería ir grabado en el fichero de configuración.
var searchEngines = new Array();
searchEngines[SEARCH_ENGINE_GOO] = new Array('http://www.goo.ne.jp/', 'http://music.goo.ne.jp', 'goo.ne.jp', 'GOO lyrics found :)', '<p class="fs16">', '</p>', 2, 'goo 音楽', 1);
searchEngines[SEARCH_ENGINE_CIX] = new Array('http://www.cix.co.kr/', 'http://www.cix.co.kr', 'cix.co.kr', 'CIX lyrics found :)', '<!-- google_ad_section_start-->', '<!-- google_ad_section_end-->', 1, 'CIX Lyrics', 3);
searchEngines[SEARCH_ENGINE_LYRICWIKI] = new Array('http://lyricwiki.org/', 'http://lyricwiki.org', 'lyricwiki.org', 'LyricWiki lyrics found :)', '<pre>', '</pre>', 0, 'LyricWiki.org', 4);
searchEngines[SEARCH_ENGINE_SOGOU] = new Array('http://mp3.sogou.com/', 'http://mp3.sogou.com', 'mp3.sogou.com', 'Sogou lyrics found :)', '<td rowspan="2" style="line-height:21px"><font class=mr>', '</font>', 1, 'Sogou', 5);
searchEngines[SEARCH_ENGINE_EVESTA] = new Array('http://www.evesta.jp/', 'http://www.evesta.jp', 'www.evesta.jp', 'evesta lyrics found :)', '<p id=\'lyric\'>', '</p>', 1, 'evesta.jp', 2);

// Variabled globales.
var searchGlobals = new Array();
searchGlobals[SEARCH_ENGINE_GOO] = '';
searchGlobals[SEARCH_ENGINE_CIX] = '';
searchGlobals[SEARCH_ENGINE_LYRICWIKI] = '';
searchGlobals[SEARCH_ENGINE_SOGOU] = '';
searchGlobals[SEARCH_ENGINE_EVESTA] = '';

var initial_artist = '';
var initial_title = '';
var global_artist = '';
var global_title = '';
var lastPriority = LOWEST_PRIORITY;
var hitsCount = 0;
var enableSTDIN = false;

// Amarok.Config.readConfig() no funciona.
var cfg_inputBoxEnabled = QFile.exists(Amarok.Info.scriptPath() + '/' + CFG_FILE_NAME);

// Añadimos el menú de configuración.
Amarok.Window.addSettingsSeparator();
Amarok.Window.addSettingsMenu("GOO_script_configure", "Configure GOO script", "configure");
var menuItem1 = Amarok.Window.SettingsMenu.GOO_script_configure;
menuItem1['triggered()'].connect(configureGOOlyrics);

// Añadimos el menú de pedir datos.
Amarok.Window.addToolsSeparator();
Amarok.Window.addToolsMenu("GOO_ask_for_lyrics", "GOO get lyrics", "kget");
var menuItem2 = Amarok.Window.ToolsMenu.GOO_ask_for_lyrics;
menuItem2['triggered()'].connect(askForLyrics);

//Amarok.Engine.trackChanged.connect(getLyrics);
Amarok.Lyrics.fetchLyrics.connect(getLyrics);
